@@ -31,14 +31,6 @@ CallManager::CallManager( QObject *parent )
     _multipartyLimit = -1;
     numRings = 0;
 
-    connectTimer = new QTimer(this);
-    connectTimer->setSingleShot(true);
-    connect( connectTimer, SIGNAL(timeout()), this, SLOT(dialingToConnected()) );
-
-    alertingTimer = new QTimer(this);
-    alertingTimer->setSingleShot(true);
-    connect( alertingTimer, SIGNAL(timeout()), this, SLOT(dialingToAlerting()) );
-
     hangupTimer = new QTimer(this);
     hangupTimer->setSingleShot(true);
     connect( hangupTimer, SIGNAL(timeout()), this, SLOT(hangupTimeout()) );
@@ -87,6 +79,13 @@ bool CallManager::command( const QString& cmd )
             return true;
         }
 
+        // For SAT test purpose, reject the dialing number 0123401234 if there
+        // is a connected call.
+        if ( hasCall( CallState_Active ) && number == "+0123401234" ) {
+            send( "NO CARRIER" );
+            return true;
+         }
+
         // If there is a connected call, place it on hold.
         changeGroup( CallState_Active, CallState_Held );
 
@@ -149,13 +148,20 @@ bool CallManager::command( const QString& cmd )
         info.dialBack = false;
         callList += info;
 
+        emit callStatesChanged( &callList );
+
         // Advertise the call state change and then return to command mode.
         sendState( info );
         send( "OK" );
 
-        // Start timers to transition the dialing call to alerting and connected.
-        alertingTimer->start(2500);
-        connectTimer->start(3000);
+        // Automatic accept of calls
+        if ( number == "6789" ) {
+            QTimer::singleShot( 1000, this, SLOT(dialingToConnected()) );
+        } else if ( number.startsWith( "05123" ) ) {
+            QTimer::singleShot( 1000, this, SLOT(dialingToConnected()) );
+        } else if ( number.startsWith( "06123" ) ) {
+            QTimer::singleShot( 1000, this, SLOT(dialingToAlerting()) );
+        }
 
     // Data call - phone number 696969
     } else if ( cmd.startsWith( "ATD" ) ) {
@@ -176,13 +182,11 @@ bool CallManager::command( const QString& cmd )
                 info.dialBack = false;
                 callList += info;
 
+                emit callStatesChanged( &callList );
+
                 // Advertise the call state change and then return to command mode.
                 sendState( info );
                 send( "CONNECT 19200" );
-
-                // Start timers to transition the dialing call to alerting and connected.
-                alertingTimer->start(2500);
-                connectTimer->start(3000);
                 } else {
                 // If not a data line
                 emit send( "NO CARRIER" );
@@ -302,7 +306,49 @@ bool CallManager::command( const QString& cmd )
     return true;
 }
 
-void CallManager::startIncomingCall( const QString& number, bool dialBack )
+void CallManager::emitRing(const CallInfo &info)
+{
+    QString str;
+
+    // Annouce the incoming call using GSM 27.007 notifications.
+    if ( info.state == CallState_Waiting ) {
+        str = "+CCWA: " + QAtUtils::encodeNumber( info.number ) + ",1,";
+
+        if (info.number.isNull())
+            str += "2";
+        else if (info.number.isEmpty())
+            str += "1";
+        else
+            str += "0";
+    } else {
+        str = "+CRING: VOICE";
+
+        if (info.number.isNull())
+            ;
+	else if (info.number.isEmpty())
+            str += "\\n\\n+CLIP: " + QAtUtils::encodeNumber( info.number ) + ",,,,1";
+        else
+            str += "\\n\\n+CLIP: " + QAtUtils::encodeNumber( info.number ) + ",,,,0";
+
+        if (info.calledNumber.isNull())
+            ;
+        else
+            str += "\\n\\n+CDIP: " + QAtUtils::encodeNumber( info.calledNumber ) + ",,,";
+
+        if (info.name.isNull())
+            ;
+	else if (info.name.isEmpty())
+            str += "\\n\\n+CNAP: \"" + info.name + "\",1";
+        else
+            str += "\\n\\n+CNAP: \"" + info.name + "\",0";
+    }
+
+    emit unsolicited(str);
+}
+
+void CallManager::startIncomingCall( const QString& number,
+                                     const QString& calledNumber,
+                                     const QString& name, bool dialBack )
 {
     // Bail out if there is already an incoming call.
     if ( idForIncoming() >= 0 ) {
@@ -320,16 +366,14 @@ void CallManager::startIncomingCall( const QString& number, bool dialBack )
         info.state = CallState_Incoming;
     }
     info.number = number;
+    info.calledNumber = calledNumber;
     info.incoming = true;
     info.dialBack = dialBack;
+    info.name = name;
     callList += info;
 
-    // Annouce the incoming call using GSM 27.007 notifications.
-    if ( info.state == CallState_Waiting ) {
-        emit unsolicited( "+CCWA: " + QAtUtils::encodeNumber( number ) + ",1" );
-    } else {
-        emit unsolicited( "RING\\n\\n+CLIP: " + QAtUtils::encodeNumber( number ) );
-    }
+    emitRing(info);
+    emit callStatesChanged( &callList );
 
     // Announce the incoming call using Ericsson-style state notifications.
     sendState( info );
@@ -339,9 +383,11 @@ void CallManager::startIncomingCall( const QString& number, bool dialBack )
     ringTimer->start(2000);
 }
 
-void CallManager::startIncomingCall( const QString& number )
+void CallManager::startIncomingCall( const QString& number,
+                                     const QString& calledNumber,
+                                     const QString& name )
 {
-    startIncomingCall( number, false );
+    startIncomingCall( number, calledNumber, name, false );
 }
 
 void CallManager::hangupAll()
@@ -351,9 +397,7 @@ void CallManager::hangupAll()
         sendState( callList[index] );
     }
     callList.clear();
-    connectTimer->stop();
-    alertingTimer->stop();
-    hangupTimer->stop();
+    emit callStatesChanged( &callList );
 }
 
 void CallManager::hangupConnected()
@@ -368,6 +412,11 @@ void CallManager::hangupConnected()
         }
     }
     callList = newCallList;
+
+    if ( !hasCall( CallState_Held ) )
+        waitingToIncoming();
+
+    emit callStatesChanged( &callList );
 }
 
 void CallManager::hangupHeld()
@@ -382,6 +431,11 @@ void CallManager::hangupHeld()
         }
     }
     callList = newCallList;
+
+    if ( !hasCall( CallState_Active ) )
+        waitingToIncoming();
+
+    emit callStatesChanged( &callList );
 }
 
 void CallManager::hangupConnectedAndHeld()
@@ -397,6 +451,8 @@ void CallManager::hangupConnectedAndHeld()
         }
     }
     callList = newCallList;
+    waitingToIncoming();
+    emit callStatesChanged( &callList );
 }
 
 void CallManager::hangupCall( int id )
@@ -404,6 +460,24 @@ void CallManager::hangupCall( int id )
     chld1x( id );
 }
 
+void CallManager::hangupRemote( int id )
+{
+    int index = indexForId( id );
+    if ( index >= 0 )
+    {
+        callList[index].state = CallState_Hangup;
+        sendState( callList[index] );
+
+        callList.removeAt( index );
+
+        if ( !hasCall( CallState_Active ) && !hasCall( CallState_Held ) )
+            waitingToIncoming();
+
+        send ( "NO CARRIER" );
+        emit callStatesChanged( &callList );
+    }
+}
+
 bool CallManager::acceptCall()
 {
     int id = idForIncoming();
@@ -418,11 +492,13 @@ bool CallManager::acceptCall()
         changeGroup( CallState_Active, CallState_Held );
         callList[index].state = CallState_Active;
         sendState( callList[index] );
+        emit callStatesChanged( &callList );
         return true;
     } else {
         // Only held calls, or no other calls, so just make the incoming call active.
         callList[index].state = CallState_Active;
         sendState( callList[index] );
+        emit callStatesChanged( &callList );
         return true;
     }
 }
@@ -452,6 +528,7 @@ bool CallManager::chld1()
         int index = indexForId(id);
         callList[index].state = CallState_Active;
         sendState( callList[index] );
+        emit callStatesChanged( &callList );
         return true;
     } else if ( hasCall( CallState_Held ) ) {
         // Hangup the active calls and activate the held ones.
@@ -460,6 +537,7 @@ bool CallManager::chld1()
             if ( callList[index].state == CallState_Held ) {
                 callList[index].state = CallState_Active;
                 sendState( callList[index] );
+                emit callStatesChanged( &callList );
             }
         }
         return true;
@@ -467,16 +545,9 @@ bool CallManager::chld1()
         // We only have active calls - hang them up.
         hangupConnected();
         return true;
-    } else if ( hasCall( CallState_Active ) ) {
-        // We only have active calls - hang them up.
-        hangupConnected();
-        return true;
     } else if ( ( id = idForDialing() ) >= 0 ) {
         // We have a dialing call.
         hangupCall(id);
-        connectTimer->stop();
-        alertingTimer->stop();
-        hangupTimer->stop();
         return true;
     } else {
         return false;
@@ -489,12 +560,6 @@ bool CallManager::chld1x( int x )
     bool found = false;
     for ( int index = 0; index < callList.size(); ++index ) {
         if ( callList[index].id == x ) {
-            if ( callList[index].state == CallState_Dialing ||
-                 callList[index].state == CallState_Alerting ) {
-                connectTimer->stop();
-                alertingTimer->stop();
-                hangupTimer->stop();
-            }
             callList[index].state = CallState_Hangup;
             sendState( callList[index] );
             found = true;
@@ -503,6 +568,11 @@ bool CallManager::chld1x( int x )
         }
     }
     callList = newCallList;
+
+    if ( !hasCall( CallState_Active ) && !hasCall( CallState_Held ) )
+        waitingToIncoming();
+
+    emit callStatesChanged( &callList );
     return found;
 }
 
@@ -522,6 +592,7 @@ bool CallManager::chld2()
         int index = indexForId( id );
         callList[index].state = CallState_Active;
         sendState( callList[index] );
+        emit callStatesChanged( &callList );
         return true;
     } else if ( hasCall( CallState_Active ) && hasCall( CallState_Held ) ) {
         // Swap the active and held calls.
@@ -572,6 +643,7 @@ bool CallManager::chld2x( int x )
             // No active calls, so make just this call active.
             callList[index].state = CallState_Active;
             sendState( callList[index] );
+            emit callStatesChanged( &callList );
         }
         return true;
     } else if ( callList[index].state == CallState_Active ) {
@@ -586,6 +658,7 @@ bool CallManager::chld2x( int x )
                     return false;
                 callList[index2].state = CallState_Held;
                 sendState( callList[index2] );
+                emit callStatesChanged( &callList );
             }
         }
         return true;
@@ -643,6 +716,7 @@ void CallManager::dialingToConnected()
     // Transition the call to its new state.
     callList[index].state = CallState_Active;
     sendState( callList[index] );
+    emit callStatesChanged( &callList );
     // If the dialed number starts with 05123, disconnect the
     // call after xx seconds, where xx is part of the dial string
     // as 05123xx
@@ -666,22 +740,45 @@ void CallManager::dialingToAlerting()
     // Transition the call to its new state.
     callList[index].state = CallState_Alerting;
     sendState( callList[index] );
+    emit callStatesChanged( &callList );
+    // If the dialed number starts with 06123, accept that
+    // call after xx seconds, where xx is part of the dial string
+    // as 06123xx
+    if( callList[index].number.startsWith( "06123" ) ) {
+        bool ok;
+        QString temp = callList[index].number;
+        temp = temp.replace( "06123" , "" );
+        int timeout = temp.toInt( &ok, 10 );
+        timeout = ok ? timeout * 1000 : 10000;
+        QTimer::singleShot( timeout, this, SLOT(dialingToConnected()) );
+    }
+}
+
+void CallManager::waitingToIncoming()
+{
+    int index = indexForId( idForState( CallState_Waiting ) );
+
+    if ( index < 0 )
+        return;
+
+    callList[index].state = CallState_Incoming;
+    sendState( callList[index] );
 }
 
 void CallManager::dialBack()
 {
-    startIncomingCall( "1234567", true );
+    startIncomingCall( "1234567", "7654321", "Alice", true );
 }
 
 void CallManager::dialBackWithHangup5()
 {
-    startIncomingCall( "1234567", true );
+    startIncomingCall( "1234567", "7654321", "Bob", true );
     hangupTimer->start( 5000 );
 }
 
 void CallManager::dialBackWithHangup4()
 {
-    startIncomingCall( "1234567", true );
+    startIncomingCall( "1234567", "7654321", "Mallory", true );
     hangupTimer->start( 4000 );
 }
 
@@ -712,7 +809,7 @@ void CallManager::sendNextRing()
         } else {
             for ( int index = 0; index < callList.size(); ++index )
                 if ( callList[index].id == idForIncoming() && callList[index].state == CallState_Incoming )
-                    emit unsolicited( "RING" );
+                    emitRing(callList[index]);
             ringTimer->start(2000);
         }
     }
@@ -797,6 +894,7 @@ void CallManager::changeGroup( CallState oldState, CallState newState )
             sendState( callList[index] );
         }
     }
+    emit callStatesChanged( &callList );
 }
 
 void CallManager::sendState( const CallInfo& info )
