@@ -29,6 +29,7 @@ public:
     SimApplicationPrivate()
     {
         expectedType = QSimCommand::NoCommand;
+        modemHandled = false;
         target = 0;
         slot = 0;
         inResponse = false;
@@ -36,6 +37,7 @@ public:
 
     SimRules   *rules;
     QSimCommand::Type expectedType;
+    bool modemHandled;
     QByteArray  currentCommand;
     QObject    *target;
     const char *slot;
@@ -86,6 +88,20 @@ void SimApplication::command( const QSimCommand& cmd,
     }
 }
 
+void SimApplication::modemHandledCommand( const QSimCommand& cmd, int timeout)
+{
+    // Record the command details, together with the type of
+    // TERMINAL RESPONSE or ENVELOPE that we expect in answer.
+    d->currentCommand = cmd.toPdu( QSimCommand::NoPduOptions );
+    d->expectedType = cmd.type();
+    d->modemHandled = true;
+
+    if ( d->rules && !d->inResponse ) {
+        d->rules->modemHandledCommandNotify( d->currentCommand );
+    }
+
+    QTimer::singleShot( timeout, this, SLOT(endSession()) );
+}
 /*!
     Sends a call control \a event to the ME.
 */
@@ -118,6 +134,7 @@ void SimApplication::start()
 void SimApplication::abort()
 {
     d->expectedType = QSimCommand::NoCommand;
+    d->modemHandled = false;
     d->currentCommand = QByteArray();
     d->target = 0;
     d->slot = 0;
@@ -162,8 +179,10 @@ bool SimApplication::envelope( const QSimEnvelope& env )
 {
     /* Process a menu selection ENVELOPE message.  We turn it into a
      * QSimTerminalResponse to make it easier to process.  */
-    if ( env.type() == QSimEnvelope::EventDownload )
+    if ( env.type() == QSimEnvelope::EventDownload ) {
+        d->rules->respond( "OK" );
         return true;
+    }
 
     if ( env.type() != QSimEnvelope::MenuSelection )
         /* Not supported */
@@ -173,6 +192,8 @@ bool SimApplication::envelope( const QSimEnvelope& env )
         /* Envelope sent for the wrong type of command. */
         return false;
 
+    d->rules->respond( "OK" );
+
     d->expectedType = QSimCommand::NoCommand;
     d->currentCommand = QByteArray();
     d->target = 0;
@@ -201,6 +222,9 @@ bool SimApplication::response( const QSimTerminalResponse& resp )
          resp.command().type() != d->expectedType )
         return false;
 
+    if (d->modemHandled == true)
+        return false;
+
     // Save the target information.
     QObject *target = d->target;
     const char *slot = d->slot;
@@ -208,6 +232,7 @@ bool SimApplication::response( const QSimTerminalResponse& resp )
     // Clear the command details, in preparation for a new command.
     if ( resp.command().type() != QSimCommand::SetupMenu ) {
         d->expectedType = QSimCommand::NoCommand;
+        d->modemHandled = false;
         d->currentCommand = QByteArray();
     }
     d->target = 0;
@@ -232,22 +257,19 @@ bool SimApplication::response( const QSimTerminalResponse& resp )
 
     // Answer the AT+CSIM command and send notification of the new command.
     if ( !d->rules )
-        return false;////
-    if ( d->currentCommand.isEmpty() || resp.command().type() == QSimCommand::SetupMenu ) {
-        // No new command, so respond with a simple OK.
-        d->rules->respond( "+CSIM: 4,9000\\n\\nOK" );
-    } else {
-        // There is a new command, so send back 91XX to the TERMINAL RESPONSE
-        // or ENVELOPE request to indicate that we have another command to
-        // be fetched.  Then send the unsolicited "*TCMD" notification.
-        QByteArray data;
-        data += (char)0x91;
-        data += (char)(d->currentCommand.size());
-        d->rules->respond( "+CSIM: " + QString::number( data.size() * 2 ) + "," +
-                           QAtUtils::toHex( data ) + "\\n\\nOK" );
-        d->rules->unsolicited
-            ( "*TCMD: " + QString::number( d->currentCommand.size() ) );
-    }
+        return false;
+
+    d->rules->respond("OK");
+
+    // No new command
+    if ( d->currentCommand.isEmpty() ||
+            resp.command().type() == QSimCommand::SetupMenu )
+        return true;
+
+    if (d->modemHandled)
+        d->rules->modemHandledCommandNotify( d->currentCommand );
+    else
+        d->rules->proactiveCommandNotify( d->currentCommand );
 
     return true;
 }
@@ -255,7 +277,13 @@ bool SimApplication::response( const QSimTerminalResponse& resp )
 void SimApplication::endSession()
 {
     d->expectedType = QSimCommand::SetupMenu;
-    d->rules->unsolicited("*TEND");
+    d->modemHandled = false;
+    d->rules->respond( "+CUSATEND", 1);
+}
+
+void SimApplication::reinitSim()
+{
+    d->rules->unsolicited( "+USIMSTATE: 1" );
 }
 
 DemoSimApplication::DemoSimApplication( SimRules *rules, QObject *parent )
@@ -288,6 +316,11 @@ const QString DemoSimApplication::getName()
 #define MainMenu_SendUSSD   13
 #define MainMenu_SendSMS    14
 #define MainMenu_Polling    15
+#define MainMenu_Timers     16
+#define MainMenu_Refresh    17
+#define MainMenu_LocalInfo  18
+#define MainMenu_BIP        19
+#define MainMenu_Handled    20
 
 #define SportsMenu_Chess        1
 #define SportsMenu_Painting     2
@@ -305,6 +338,7 @@ const QString DemoSimApplication::getName()
 #define SendSSMenu_CLIR     5
 #define SendSSMenu_CoLP     6
 #define SendSSMenu_CoLR     7
+#define SendSSMenu_CNAP     8
 
 #define CBMenu_Activation       1
 #define CBMenu_Interrogation    2
@@ -320,21 +354,17 @@ const QString DemoSimApplication::getName()
 #define CWMenu_Interrogation    2
 #define CWMenu_Deactivation     3
 
-#define CLIPMenu_Activation       1
-#define CLIPMenu_Interrogation    2
-#define CLIPMenu_Deactivation     3
+#define CLIPMenu_Interrogation    1
 
 #define CLIRMenu_Activation       1
 #define CLIRMenu_Interrogation    2
 #define CLIRMenu_Deactivation     3
 
-#define CoLPMenu_Activation       1
-#define CoLPMenu_Interrogation    2
-#define CoLPMenu_Deactivation     3
+#define CoLPMenu_Interrogation    1
 
-#define CoLRMenu_Activation       1
-#define CoLRMenu_Interrogation    2
-#define CoLRMenu_Deactivation     3
+#define CoLRMenu_Interrogation    1
+
+#define CNAPMenu_Interrogation    1
 
 #define Language_Specific       1
 #define Language_Non_Specific   2
@@ -346,6 +376,20 @@ const QString DemoSimApplication::getName()
 #define SendUSSD_Error      4
 #define SendUSSD_Main       5
 
+#define LocalInfoMenu_Time  1
+#define LocalInfoMenu_Lang  2
+
+#define BIPMenu_OpenChannel_TCP        1
+#define BIPMenu_OpenChannel_UDP        2
+#define BIPMenu_CloseChannel           3
+#define BIPMenu_ReceiveData            4
+#define BIPMenu_SendData_Immediately   5
+#define BIPMenu_SendData_Store         6
+#define BIPMenu_GetChannelStatus       7
+
+#define Handled_SendSMS	1
+#define Handled_InitNAA 2
+
 enum SendSMSMenuItems {
 	SendSMS_Unpacked = 1,
 	SendSMS_Packed,
@@ -358,6 +402,13 @@ enum PollingMenuItems {
 	Polling_30s,
 };
 
+enum TimersMenuItems {
+	Timers_Start = 1,
+	Timers_Stop,
+	Timers_Sleep,
+	Timers_Query,
+};
+
 void DemoSimApplication::mainMenu()
 {
     QSimCommand cmd;
@@ -365,6 +416,7 @@ void DemoSimApplication::mainMenu()
     QList<QSimMenuItem> items;
 
     cmd.setType( QSimCommand::SetupMenu );
+    cmd.setTitle( "Phonesim services" );
 
     item.setIdentifier( MainMenu_News );
     item.setLabel( "News" );
@@ -426,6 +478,26 @@ void DemoSimApplication::mainMenu()
     item.setLabel( "SIM Polling" );
     items += item;
 
+    item.setIdentifier( MainMenu_Timers );
+    item.setLabel( "Timers" );
+    items += item;
+
+    item.setIdentifier( MainMenu_Refresh );
+    item.setLabel( "SIM Refresh" );
+    items += item;
+
+    item.setIdentifier( MainMenu_LocalInfo );
+    item.setLabel( "Provide Local Information" );
+    items += item;
+
+    item.setIdentifier( MainMenu_BIP );
+    item.setLabel( "BIP commands" );
+    items += item;
+
+    item.setIdentifier( MainMenu_Handled );
+    item.setLabel( "Modem-Handled commands" );
+    items += item;
+
     cmd.setMenuItems( items );
 
     command( cmd, 0, 0 );
@@ -550,6 +622,36 @@ void DemoSimApplication::mainMenuSelection( int id )
         }
         break;
 
+        case MainMenu_Timers:
+        {
+            sendTimersMenu();
+        }
+        break;
+
+        case MainMenu_Refresh:
+        {
+            sendRefreshMenu();
+        }
+        break;
+
+        case MainMenu_LocalInfo:
+        {
+            sendLocalInfoMenu();
+        }
+        break;
+
+        case MainMenu_BIP:
+        {
+            sendBIPMenu();
+        }
+        break;
+
+        case MainMenu_Handled:
+        {
+            sendHandledMenu();
+        }
+        break;
+
         default:
         {
             // Don't know what this item is, so just re-display the main menu.
@@ -860,6 +962,7 @@ void DemoSimApplication::toneMenu( const QSimTerminalResponse& resp )
     if ( resp.result() == QSimTerminalResponse::Success ) {
         // Item selected.
         cmd.setType( QSimCommand::PlayTone );
+        cmd.setText( "Play a standard supervisory tone" );
         cmd.setDestinationDevice( QSimCommand::Earpiece );
         cmd.setTone( (QSimCommand::Tone)( resp.menuItem() ) );
         if ( cmd.tone() == QSimCommand::ToneDial )
@@ -1111,7 +1214,11 @@ void DemoSimApplication::sendSendSSMenu()
     items += item;
 
     item.setIdentifier( SendSSMenu_CLIP );
-    item.setLabel( "CLIP (Calling Line Idnetification Presentation)" );
+    item.setLabel( "CLIP (Calling Line Identification Presentation)" );
+    items += item;
+
+    item.setIdentifier( SendSSMenu_CNAP );
+    item.setLabel( "CNAP (Calling Name Identification Presentation)" );
     items += item;
 
     item.setIdentifier( SendSSMenu_CLIR );
@@ -1306,6 +1413,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "**62*+155543*11#" );
+                cmd.setText( "Sending SS Registration" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1315,6 +1423,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*62#" );
+                cmd.setText( "Sending SS Activation" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1324,6 +1433,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*#62**11#" );
+                cmd.setText( "Sending SS Interrogation class voice" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1333,6 +1443,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "#62#" );
+                cmd.setText( "Sending SS Deactivation" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1342,6 +1453,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "##62#" );
+                cmd.setText( "Sending SS Erasure" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1437,18 +1549,10 @@ void DemoSimApplication::sendCLIPMenu()
     cmd.setType( QSimCommand::SelectItem );
     cmd.setTitle( "CLIP" );
 
-    item.setIdentifier( CLIPMenu_Activation );
-    item.setLabel( "Activation" );
-    items += item;
-
     item.setIdentifier( CLIPMenu_Interrogation );
     item.setLabel( "Interrogation" );
     items += item;
 
-    item.setIdentifier( CLIPMenu_Deactivation );
-    item.setLabel( "Deactivation" );
-    items += item;
-
     cmd.setMenuItems( items );
 
     command( cmd, this, SLOT(CLIPMenu(QSimTerminalResponse)) );
@@ -1460,33 +1564,14 @@ void DemoSimApplication::CLIPMenu( const QSimTerminalResponse& resp )
 
     if ( resp.result() == QSimTerminalResponse::Success ) {
         switch ( resp.menuItem() ) {
-
-            case CLIPMenu_Activation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "*30#" );
-                command( cmd, this, SLOT(sendCLIPMenu()) );
-            }
-            break;
-
             case CLIPMenu_Interrogation:
             {
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*#30#" );
                 command( cmd, this, SLOT(sendCLIPMenu()) );
+                break;
             }
-            break;
-
-            case CLIPMenu_Deactivation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "#30#" );
-                command( cmd, this, SLOT(sendCLIPMenu()) );
-            }
-            break;
 
             default:
                 endSession();
@@ -1579,18 +1664,10 @@ void DemoSimApplication::sendCoLPMenu()
     cmd.setType( QSimCommand::SelectItem );
     cmd.setTitle( "CoLP" );
 
-    item.setIdentifier( CoLPMenu_Activation );
-    item.setLabel( "Activation" );
-    items += item;
-
     item.setIdentifier( CoLPMenu_Interrogation );
     item.setLabel( "Interrogation" );
     items += item;
 
-    item.setIdentifier( CoLPMenu_Deactivation );
-    item.setLabel( "Deactivation" );
-    items += item;
-
     cmd.setMenuItems( items );
 
     command( cmd, this, SLOT(CoLPMenu(QSimTerminalResponse)) );
@@ -1602,33 +1679,14 @@ void DemoSimApplication::CoLPMenu( const QSimTerminalResponse& resp )
 
     if ( resp.result() == QSimTerminalResponse::Success ) {
         switch ( resp.menuItem() ) {
-
-            case CoLPMenu_Activation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "*76#" );
-                command( cmd, this, SLOT(sendCoLPMenu()) );
-            }
-            break;
-
             case CoLPMenu_Interrogation:
             {
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*#76#" );
                 command( cmd, this, SLOT(sendCoLPMenu()) );
+                break;
             }
-            break;
-
-            case CoLPMenu_Deactivation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "#76#" );
-                command( cmd, this, SLOT(sendCoLPMenu()) );
-            }
-            break;
 
             default:
                 endSession();
@@ -1650,18 +1708,10 @@ void DemoSimApplication::sendCoLRMenu()
     cmd.setType( QSimCommand::SelectItem );
     cmd.setTitle( "CoLR" );
 
-    item.setIdentifier( CoLRMenu_Activation );
-    item.setLabel( "Activation" );
-    items += item;
-
     item.setIdentifier( CoLRMenu_Interrogation );
     item.setLabel( "Interrogation" );
     items += item;
 
-    item.setIdentifier( CoLRMenu_Deactivation );
-    item.setLabel( "Deactivation" );
-    items += item;
-
     cmd.setMenuItems( items );
 
     command( cmd, this, SLOT(CoLRMenu(QSimTerminalResponse)) );
@@ -1673,33 +1723,58 @@ void DemoSimApplication::CoLRMenu( const QSimTerminalResponse& resp )
 
     if ( resp.result() == QSimTerminalResponse::Success ) {
         switch ( resp.menuItem() ) {
-
-            case CoLRMenu_Activation:
-            {
-                cmd.setType( QSimCommand::SendSS );
-                cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "*77#" );
-                command( cmd, this, SLOT(sendCoLRMenu()) );
-            }
-            break;
-
             case CoLRMenu_Interrogation:
             {
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*#77#" );
                 command( cmd, this, SLOT(sendCoLRMenu()) );
+                break;
             }
-            break;
 
-            case CoLRMenu_Deactivation:
+            default:
+                endSession();
+                break;
+        }
+    } else if ( resp.result() == QSimTerminalResponse::BackwardMove ) {
+        sendSendSSMenu();
+    } else {
+        endSession();
+    }
+}
+
+void DemoSimApplication::sendCNAPMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "CNAP" );
+
+    item.setIdentifier( CNAPMenu_Interrogation );
+    item.setLabel( "Interrogation" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(CNAPMenu(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::CNAPMenu( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() == QSimTerminalResponse::Success ) {
+        switch ( resp.menuItem() ) {
+            case CNAPMenu_Interrogation:
             {
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
-                cmd.setNumber( "#77#" );
-                command( cmd, this, SLOT(sendCoLRMenu()) );
+                cmd.setNumber( "*#300#" );
+                command( cmd, this, SLOT(sendCNAPMenu()) );
+                break;
             }
-            break;
 
             default:
                 endSession();
@@ -2044,3 +2119,603 @@ void DemoSimApplication::pollingMenuResp( const QSimTerminalResponse& resp )
         break;
     }
 }
+
+void DemoSimApplication::sendTimersMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "Timer ops" );
+
+    item.setIdentifier( Timers_Start );
+    item.setLabel( "Reset timer 1 to 1h" );
+    items += item;
+
+    item.setIdentifier( Timers_Stop );
+    item.setLabel( "Stop all timers" );
+    items += item;
+
+    item.setIdentifier( Timers_Sleep );
+    item.setLabel( "Sleep for 10s using timer 2" );
+    items += item;
+
+    item.setIdentifier( Timers_Query );
+    item.setLabel( "Show statuses" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(timersMenuResp(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::timersMenuResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    /* Item selected. */
+    switch ( resp.menuItem() ) {
+    case Timers_Start:
+        cmd.setQualifier( 0 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( 1 );
+        cmd.setDuration( 3600000 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(endSession()) );
+        break;
+
+    case Timers_Stop:
+        cmd.setQualifier( 1 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( 1 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(timersCmdResp(QSimTerminalResponse)) );
+        break;
+
+    case Timers_Sleep:
+        cmd.setQualifier( 0 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( 2 );
+        cmd.setDuration( 10000 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, NULL, NULL );
+        break;
+
+    case Timers_Query:
+        cmd.setQualifier( 2 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( 1 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(timersCmdResp(QSimTerminalResponse)) );
+	timerStatus = "";
+        break;
+    }
+}
+
+void DemoSimApplication::timersCmdResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.command().qualifier() == 1 ) {
+        if ( resp.command().timerId() < 1 ) {
+            endSession();
+            return;
+        }
+
+        /* Stop the next timer */
+        cmd.setQualifier( 1 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( resp.timerId() + 1 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+
+        if ( cmd.timerId() >= 8 )
+            command( cmd, this, SLOT(endSession()) );
+        else
+            command( cmd, this, SLOT(timersCmdResp(QSimTerminalResponse)) );
+
+        return;
+    }
+
+    if ( resp.command().qualifier() == 2 ) {
+        if ( resp.result() == QSimTerminalResponse::Success ) {
+            QString status;
+            status.sprintf( "Timer %i expires in %i:%02i:%02i.\n",
+                    resp.timerId(), resp.duration() / 3600000,
+                    (resp.duration() / 60000) % 60,
+                    (resp.duration() / 1000) % 60 );
+
+            timerStatus += status;
+        }
+
+        if ( resp.timerId() == 8 || resp.timerId() < 1 ) {
+            /* All timers done */
+            if ( timerStatus.length() == 0 )
+                timerStatus = "All timers are stopped.";
+
+            cmd.setType( QSimCommand::DisplayText );
+            cmd.setDestinationDevice( QSimCommand::Display );
+            cmd.setText( timerStatus.left(220) );
+            command( cmd, this, SLOT(endSession()) );
+            return;
+        }
+
+        /* Interrogate the next timer */
+        cmd.setQualifier( 2 );
+        cmd.setType( QSimCommand::TimerManagement );
+        cmd.setTimerId( resp.timerId() + 1 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        command( cmd, this, SLOT(timersCmdResp(QSimTerminalResponse)) );
+
+        return;
+    }
+}
+
+bool DemoSimApplication::envelope( const QSimEnvelope& env )
+{
+    if ( env.type() != QSimEnvelope::TimerExpiration )
+        return SimApplication::envelope( env );
+
+    QSimCommand cmd;
+
+    cmd.setType( QSimCommand::DisplayText );
+    cmd.setDestinationDevice( QSimCommand::Display );
+    cmd.setText( "Timer expired." );
+    command( cmd, this, SLOT(endSession()) );
+
+    return true;
+}
+
+void DemoSimApplication::sendRefreshMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "SIM Refresh menu" );
+
+    /* Use qualifier value + 1 for id */
+    item.setIdentifier( 1 );
+    item.setLabel( "NAA Initialization+Full File Change Notification" );
+    items += item;
+
+    item.setIdentifier( 2 );
+    item.setLabel( "File Change Notification (EFmsisdn,EFecc,EFfdn)" );
+    items += item;
+
+    item.setIdentifier( 3 );
+    item.setLabel( "NAA Initialization+File Change Notification" );
+    items += item;
+
+    item.setIdentifier( 4 );
+    item.setLabel( "NAA Initialization" );
+    items += item;
+
+    item.setIdentifier( 5 );
+    item.setLabel( "UICC Reset" );
+    items += item;
+
+    item.setIdentifier( 6 );
+    item.setLabel( "NAA Application Reset" );
+    items += item;
+
+    item.setIdentifier( 7 );
+    item.setLabel( "NAA Session Reset" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(refreshMenuResp(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::refreshMenuResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    /* Item selected. */
+
+    cmd.setType( QSimCommand::Refresh );
+    cmd.setQualifier( resp.menuItem() - 1 );
+    cmd.setDestinationDevice( QSimCommand::ME );
+    cmd.setText( "" );
+
+    if ( cmd.refreshType() == QSimCommand::FileChange ||
+            cmd.refreshType() == QSimCommand::InitAndFileChange ||
+            cmd.refreshType() == QSimCommand::NaaSessionReset ) {
+        QByteArray files;
+        files += (char) 0x03;
+        /* EFmsisdn */
+        files += (char) 0x3f;
+        files += (char) 0x00;
+        files += (char) 0x7f;
+        files += (char) 0xff;
+        files += (char) 0x6f;
+        files += (char) 0x40;
+        /* EFecc */
+        files += (char) 0x3f;
+        files += (char) 0x00;
+        files += (char) 0x7f;
+        files += (char) 0xff;
+        files += (char) 0x6f;
+        files += (char) 0xb7;
+        /* EFfdn */
+        files += (char) 0x3f;
+        files += (char) 0x00;
+        files += (char) 0x7f;
+        files += (char) 0xff;
+        files += (char) 0x6f;
+        files += (char) 0x3b;
+        cmd.addExtensionField( 0x92, files );
+    }
+
+    command( cmd, this, SLOT(endSession()) );
+
+    if (cmd.refreshType() != QSimCommand::FileChange)
+        QTimer::singleShot( 1000, this, SLOT(reinitSim()) );
+}
+
+void DemoSimApplication::sendLocalInfoMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "Provide Local Information" );
+
+    item.setIdentifier( LocalInfoMenu_Time );
+    item.setLabel( "Date, time and time zone" );
+    items += item;
+
+    item.setIdentifier( LocalInfoMenu_Lang );
+    item.setLabel( "Language setting" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(localInfoMenu(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::localInfoMenu( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() == QSimTerminalResponse::Success ) {
+        switch ( resp.menuItem() ) {
+
+            case LocalInfoMenu_Time:
+            {
+                cmd.setType( QSimCommand::ProvideLocalInformation );
+                cmd.setQualifier( 0x03 );
+                command( cmd, this, SLOT(sendLocalInfoMenu()) );
+            }
+            break;
+
+            case LocalInfoMenu_Lang:
+            {
+                cmd.setType( QSimCommand::ProvideLocalInformation );
+                cmd.setQualifier( 0x04 );
+                command( cmd, this, SLOT(sendLocalInfoMenu()) );
+            }
+            break;
+
+            default:
+                endSession();
+                break;
+        }
+    } else {
+        endSession();
+    }
+}
+
+void DemoSimApplication::BIPMenu( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() == QSimTerminalResponse::Success ) {
+	switch ( resp.menuItem() ) {
+
+            case BIPMenu_OpenChannel_TCP:
+            {
+                QByteArray bearerDesc;
+                QByteArray uti;
+                QByteArray destAddress;
+                QByteArray apn;
+
+                bearerDesc += 0x02;
+                bearerDesc += 0x03;
+                bearerDesc += 0x04;
+                bearerDesc += 0x03;
+                bearerDesc += 0x04;
+                bearerDesc += 0x1F;
+                bearerDesc += 0x02;
+
+                uti += 0x02; /* TCP, UICC in client mode, remote connection */
+                uti += 0x30;
+                uti += 0x3C; /* port 12348 */
+
+                destAddress += 0x21;
+                destAddress += 0x7F; /* local host 127.0.0.1 */
+                destAddress += (char) 0x00;
+                destAddress += (char) 0x00;
+                destAddress += 0x01;
+
+                apn += 0x06;
+                apn += 0x54;
+                apn += 0x65;
+                apn += 0x73;
+                apn += 0x74;
+                apn += 0x47;
+                apn += 0x70;
+                apn += 0x02;
+                apn += 0x72;
+                apn += 0x73;
+
+                cmd.setType( QSimCommand::OpenChannel );
+                cmd.setQualifier( QSimCommand::OpenChannelImmediate );
+                cmd.setText("Open channel TCP");
+                cmd.setBearerDesc(bearerDesc);
+                cmd.setBufferSize(1400);
+                cmd.setApn(apn);
+                cmd.setUserLogin("UserLog");
+                cmd.setUserPassword("UserPwd");
+                cmd.setUti(uti);
+                cmd.setDestAddress(destAddress);
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_OpenChannel_UDP:
+            {
+                QByteArray bearerDesc;
+                QByteArray uti;
+                QByteArray destAddress;
+                QByteArray apn;
+
+                bearerDesc += 0x02;
+                bearerDesc += 0x03;
+                bearerDesc += 0x04;
+                bearerDesc += 0x03;
+                bearerDesc += 0x04;
+                bearerDesc += 0x1F;
+                bearerDesc += 0x02;
+
+                uti += 0x01; /* UDP, UICC in client mode, remote connection */
+                uti += 0x30;
+                uti += 0x3C; /* port 12348 */
+
+                destAddress += 0x21;
+                destAddress += 0x7F; /* local host 127.0.0.1 */
+                destAddress += (char) 0x00;
+                destAddress += (char) 0x00;
+                destAddress += 0x01;
+
+                apn += 0x06;
+                apn += 0x54;
+                apn += 0x65;
+                apn += 0x73;
+                apn += 0x74;
+                apn += 0x47;
+                apn += 0x70;
+                apn += 0x02;
+                apn += 0x72;
+                apn += 0x73;
+
+                cmd.setType( QSimCommand::OpenChannel );
+                cmd.setQualifier( QSimCommand::OpenChannelImmediate );
+                cmd.setText("Open channel UDP");
+                cmd.setBearerDesc(bearerDesc);
+                cmd.setBufferSize(1400);
+                cmd.setApn(apn);
+                cmd.setUserLogin("UserLog");
+                cmd.setUserPassword("UserPwd");
+                cmd.setUti(uti);
+                cmd.setDestAddress(destAddress);
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_ReceiveData:
+            {
+                cmd.setType( QSimCommand::ReceiveData );
+                cmd.setDestinationDevice( QSimCommand::Channel1 );
+                cmd.setText("Receive Data 1");
+                cmd.setDataLength(200);
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_SendData_Immediately:
+            {
+                QByteArray data;
+
+                data += 0x61;
+                data += 0x62;
+                data += 0x63;
+                data += 0x64;
+                data += 0x65;
+
+                cmd.setQualifier( QSimCommand::SendDataImmediately );
+                cmd.setType( QSimCommand::SendData );
+                cmd.setDestinationDevice( QSimCommand::Channel1 );
+                cmd.setText("Send Data Immediately");
+                cmd.addExtensionField( 0x36, data );
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_SendData_Store:
+            {
+                QByteArray data;
+
+                data += 0x31;
+                data += 0x32;
+                data += 0x33;
+                data += 0x34;
+                data += 0x35;
+
+                cmd.setQualifier( QSimCommand::SendDataStoreData );
+                cmd.setType( QSimCommand::SendData );
+                cmd.setDestinationDevice( QSimCommand::Channel1 );
+                cmd.setText("Store Data");
+                cmd.addExtensionField( 0x36, data );
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_CloseChannel:
+            {
+                cmd.setType( QSimCommand::CloseChannel );
+                cmd.setDestinationDevice( QSimCommand::Channel1 );
+                cmd.setText("Close ID 1");
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            case BIPMenu_GetChannelStatus:
+            {
+                cmd.setType( QSimCommand::GetChannelStatus );
+                command( cmd, this, SLOT(sendBIPMenu()) );
+            }
+            break;
+
+            default:
+                endSession();
+                break;
+        }
+    } else {
+        endSession();
+    }
+}
+
+void DemoSimApplication::sendBIPMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "BIP commands Menu" );
+
+    item.setIdentifier( BIPMenu_OpenChannel_TCP );
+    item.setLabel( "Open channel - TCP in client mode" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_OpenChannel_UDP );
+    item.setLabel( "Open channel - UDP in client mode" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_CloseChannel );
+    item.setLabel( "Close channel" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_ReceiveData );
+    item.setLabel( "Receive data" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_SendData_Immediately );
+    item.setLabel( "Send data - Immediately" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_SendData_Store );
+    item.setLabel( "Send data - Store" );
+    items += item;
+
+    item.setIdentifier( BIPMenu_GetChannelStatus );
+    item.setLabel( "Get channel status" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(BIPMenu(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::sendHandledMenu()
+{
+    QSimCommand cmd;
+    QSimMenuItem item;
+    QList<QSimMenuItem> items;
+
+    cmd.setType( QSimCommand::SelectItem );
+    cmd.setTitle( "Modem-Handled Commands Menu" );
+
+    item.setIdentifier( Handled_SendSMS );
+    item.setLabel( "Send Modem-Handled SMS" );
+    items += item;
+
+    item.setIdentifier( Handled_InitNAA );
+    item.setLabel( "NAA Initialization+Full File Change Notification" );
+    items += item;
+
+    cmd.setMenuItems( items );
+
+    command( cmd, this, SLOT(handledMenuResp(QSimTerminalResponse)) );
+}
+
+void DemoSimApplication::handledMenuResp( const QSimTerminalResponse& resp )
+{
+    QSimCommand cmd;
+
+    if ( resp.result() != QSimTerminalResponse::Success ) {
+        /* Unknown response - just go back to the main menu. */
+        endSession();
+
+        return;
+    }
+
+    /* Item selected. */
+    switch ( resp.menuItem() ) {
+    case Handled_SendSMS:
+    {
+        QSMSMessage sms;
+        sms.setValidityPeriod( -1 );
+        sms.setMessageClass( 2 );
+        sms.setProtocol( 0 );
+        sms.setRecipient( "12345" );
+        sms.setText( smsText );
+        sms.setForceGsm( false );
+        sms.setBestScheme( QSMS_8BitAlphabet );
+        sms.setDataCodingScheme( 0xf6 );
+
+        cmd.setType( QSimCommand::SendSMS );
+        cmd.setText( "Sending an SMS to our friends at 12345" );
+        cmd.setNumber( "123" );
+        cmd.addExtensionField( 0x8b, sms.toPdu().mid( 1 ) );
+        cmd.setDestinationDevice( QSimCommand::Network );
+
+        modemHandledCommand(cmd, 6000);
+        break;
+    }
+
+    case Handled_InitNAA:
+    {
+        cmd.setType( QSimCommand::Refresh );
+        cmd.setQualifier( 3 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        cmd.setText( "" );
+
+        modemHandledCommand(cmd, 1000);
+        QTimer::singleShot( 1100, this, SLOT(reinitSim()) );
+        break;
+    }
+
+    default:
+	endSession();
+	break;
+    }
+}
